@@ -3,15 +3,19 @@
 """Unit tests for M2Crypto.EC, ECDSA part.
 
 Copyright (c) 2000 Ng Pheng Siong. All rights reserved.
-Portions copyright (c) 2005-2006 Vrije Universiteit Amsterdam. All rights reserved.
+Portions copyright (c) 2005-2006 Vrije Universiteit Amsterdam. All
+rights reserved.
 """
 import hashlib
-try:
-    import unittest2 as unittest
-except ImportError:
-    import unittest
+import logging
+
+from M2Crypto import EC, Rand
+
+from tests import unittest
+from tests.test_ec_curves import tested_curve
+
+log = logging.getLogger(__name__)
 
-from M2Crypto import EC, BIO, Rand, m2
 
 class ECDSATestCase(unittest.TestCase):
 
@@ -19,7 +23,7 @@ class ECDSATestCase(unittest.TestCase):
     privkey = 'tests/ec.priv.pem'
     pubkey = 'tests/ec.pub.pem'
 
-    data = hashlib.sha1('Can you spell subliminal channel?').digest()
+    data = hashlib.sha1(b'Can you spell subliminal channel?').digest()
 
     def callback(self, *args):
         pass
@@ -33,17 +37,17 @@ class ECDSATestCase(unittest.TestCase):
 
     def test_loadkey(self):
         ec = EC.load_key(self.privkey)
-        self.assertEqual(len(ec), 233)
+        self.assertEqual(len(ec), tested_curve[1])
 
     def test_loadpubkey(self):
         # XXX more work needed
         ec = EC.load_pub_key(self.pubkey)
-        self.assertEqual(len(ec), 233)
+        self.assertEqual(len(ec), tested_curve[1])
         with self.assertRaises(EC.ECError):
             EC.load_pub_key(self.errkey)
 
     def _test_sign_dsa(self):
-        ec = EC.gen_params(EC.NID_sect233k1)
+        ec = EC.gen_params(tested_curve[0])
         # ec.gen_key()
         with self.assertRaises(EC.ECError):
             ec.sign_dsa(self.data)
@@ -67,8 +71,19 @@ class ECDSATestCase(unittest.TestCase):
         assert not ec2.verify_dsa(self.data, s, r)
 
     def test_genparam(self):
-        ec = EC.gen_params(EC.NID_sect233k1)
-        self.assertEqual(len(ec), 233)
+        ec = EC.gen_params(tested_curve[0])
+        self.assertEqual(len(ec), tested_curve[1])
+
+    def test_pub_key_from_params(self):
+        curve = EC.NID_prime256v1
+        ec = EC.gen_params(curve)
+        ec.gen_key()
+        ec_pub = ec.pub()
+        k = ec_pub.get_key()
+        ec2 = EC.pub_key_from_params(curve, k)
+        assert ec2.check_key()
+        r, s = ec.sign_dsa(self.data)
+        assert ec2.verify_dsa(self.data, r, s)
 
 
 def suite():
