@@ -1,19 +1,20 @@
 #!/usr/bin/env python
+from __future__ import absolute_import
 
 """Unit tests for M2Crypto.RSA.
 
 Copyright (c) 2000 Ng Pheng Siong. All rights reserved."""
 
 import hashlib
+import logging
 import os
-try:
-    import unittest2 as unittest
-except ImportError:
-    import unittest
 
-from M2Crypto import BIO, RSA, Rand, X509, m2
+from M2Crypto import BIO, RSA, Rand, X509, m2, six
+from tests import unittest
+from tests.fips import fips_mode
+
+log = logging.getLogger('test_RSA')
 
-from fips import fips_mode
 
 class RSATestCase(unittest.TestCase):
 
@@ -22,7 +23,7 @@ class RSATestCase(unittest.TestCase):
     privkey2 = 'tests/rsa.priv2.pem'
     pubkey = 'tests/rsa.pub.pem'
 
-    data = hashlib.sha1('The magic words are squeamish ossifrage.').digest()
+    data = hashlib.sha1(b'The magic words are squeamish ossifrage.').digest()
 
     e_padding_ok = ('pkcs1_padding', 'pkcs1_oaep_padding')
 
@@ -37,11 +38,15 @@ class RSATestCase(unittest.TestCase):
 
     def pp_callback(self, *args):
         # The passphrase for rsa.priv2.pem is 'qwerty'.
-        return 'qwerty'
+        return b'qwerty'
 
     def pp2_callback(self, *args):
         # Misbehaving passphrase callback.
-        pass
+        return b'blabla'
+
+    def test_rsa_exceptions(self):
+        with self.assertRaises(RSA.RSAError):
+            RSA.rsa_error()
 
     def test_loadkey_junk(self):
         with self.assertRaises(RSA.RSAError):
@@ -51,7 +56,7 @@ class RSATestCase(unittest.TestCase):
         rsa = RSA.load_key(self.privkey2, self.pp_callback)
         self.assertEqual(len(rsa), 1024)
         self.assertEqual(rsa.e,
-                         '\000\000\000\003\001\000\001')  # aka 65537 aka 0xf4
+                         b'\000\000\000\003\001\000\001')  # aka 65537 aka 0xf4
         self.assertEqual(rsa.check_key(), 1)
 
     def test_loadkey_pp_bad_cb(self):
@@ -62,32 +67,33 @@ class RSATestCase(unittest.TestCase):
         rsa = RSA.load_key(self.privkey)
         self.assertEqual(len(rsa), 1024)
         self.assertEqual(rsa.e,
-                         '\000\000\000\003\001\000\001')  # aka 65537 aka 0xf4
-        self.assertEqual(rsa.n, "\x00\x00\x00\x81\x00\xcde!\x15\xdah\xb5`\xce[\xd6\x17d\xba8\xc1I\xb1\xf1\xber\x86K\xc7\xda\xb3\x98\xd6\xf6\x80\xae\xaa\x8f!\x9a\xefQ\xdeh\xbb\xc5\x99\x01o\xebGO\x8e\x9b\x9a\x18\xfb6\xba\x12\xfc\xf2\x17\r$\x00\xa1\x1a \xfc/\x13iUm\x04\x13\x0f\x91D~\xbf\x08\x19C\x1a\xe2\xa3\x91&\x8f\xcf\xcc\xf3\xa4HRf\xaf\xf2\x19\xbd\x05\xe36\x9a\xbbQ\xc86|(\xad\x83\xf2Eu\xb2EL\xdf\xa4@\x7f\xeel|\xfcU\x03\xdb\x89'")
+                         b'\000\000\000\003\001\000\001')  # aka 65537 aka 0xf4
+        self.assertEqual(rsa.n, b"\x00\x00\x00\x81\x00\xcde!\x15\xdah\xb5`\xce[\xd6\x17d\xba8\xc1I\xb1\xf1\xber\x86K\xc7\xda\xb3\x98\xd6\xf6\x80\xae\xaa\x8f!\x9a\xefQ\xdeh\xbb\xc5\x99\x01o\xebGO\x8e\x9b\x9a\x18\xfb6\xba\x12\xfc\xf2\x17\r$\x00\xa1\x1a \xfc/\x13iUm\x04\x13\x0f\x91D~\xbf\x08\x19C\x1a\xe2\xa3\x91&\x8f\xcf\xcc\xf3\xa4HRf\xaf\xf2\x19\xbd\x05\xe36\x9a\xbbQ\xc86|(\xad\x83\xf2Eu\xb2EL\xdf\xa4@\x7f\xeel|\xfcU\x03\xdb\x89'")
         with self.assertRaises(AttributeError):
             getattr(rsa, 'nosuchprop')
         self.assertEqual(rsa.check_key(), 1)
 
     def test_loadkey_bio(self):
-        keybio = BIO.MemoryBuffer(open(self.privkey).read())
+        with open(self.privkey, "rb") as f:
+            keybio = BIO.MemoryBuffer(f.read())
         rsa = RSA.load_key_bio(keybio)
         self.assertEqual(len(rsa), 1024)
         self.assertEqual(rsa.e,
-                         '\000\000\000\003\001\000\001')  # aka 65537 aka 0xf4
+                         b'\000\000\000\003\001\000\001')  # aka 65537 aka 0xf4
         self.assertEqual(rsa.check_key(), 1)
 
     def test_keygen(self):
         rsa = RSA.gen_key(1024, 65537, self.gen_callback)
         self.assertEqual(len(rsa), 1024)
         self.assertEqual(rsa.e,
-                         '\000\000\000\003\001\000\001')  # aka 65537 aka 0xf4
+                         b'\000\000\000\003\001\000\001')  # aka 65537 aka 0xf4
         self.assertEqual(rsa.check_key(), 1)
 
     def test_keygen_bad_cb(self):
         rsa = RSA.gen_key(1024, 65537, self.gen2_callback)
         self.assertEqual(len(rsa), 1024)
         self.assertEqual(rsa.e,
-                         '\000\000\000\003\001\000\001')  # aka 65537 aka 0xf4
+                         b'\000\000\000\003\001\000\001')  # aka 65537 aka 0xf4
         self.assertEqual(rsa.check_key(), 1)
 
     def test_private_encrypt(self):
@@ -107,6 +113,8 @@ class RSATestCase(unittest.TestCase):
         with self.assertRaises(TypeError):
             priv.private_encrypt(self.gen_callback, RSA.pkcs1_padding)
 
+    @unittest.skipIf(m2.OPENSSL_VERSION_NUMBER < 0x1010103f,
+                     'Relies on fix which happened only in OpenSSL 1.1.1c')
     def test_public_encrypt(self):
         priv = RSA.load_key(self.privkey)
         # pkcs1_padding, pkcs1_oaep_padding
@@ -118,13 +126,11 @@ class RSATestCase(unittest.TestCase):
 
         # sslv23_padding
         ctxt = priv.public_encrypt(self.data, RSA.sslv23_padding)
-        with self.assertRaises(RSA.RSAError):
-            priv.private_decrypt(ctxt, RSA.sslv23_padding)
-        with self.assertRaises(RSA.RSAError):
-            priv.private_decrypt(ctxt, RSA.sslv23_padding)
+        res = priv.private_decrypt(ctxt, RSA.sslv23_padding)
+        self.assertEqual(res, self.data)
 
         # no_padding
-        with self.assertRaises(RSA.RSAError):
+        with six.assertRaisesRegex(self, RSA.RSAError, 'data too small'):
             priv.public_encrypt(self.data, RSA.no_padding)
 
         # Type-check the data to be encrypted.
@@ -134,13 +140,13 @@ class RSATestCase(unittest.TestCase):
     def test_x509_public_encrypt(self):
         x509 = X509.load_cert("tests/recipient.pem")
         rsa = x509.get_pubkey().get_rsa()
-        rsa.public_encrypt("data", RSA.pkcs1_padding)
+        rsa.public_encrypt(b"data", RSA.pkcs1_padding)
 
     def test_loadpub(self):
         rsa = RSA.load_pub_key(self.pubkey)
         self.assertEqual(len(rsa), 1024)
         self.assertEqual(rsa.e,
-                         '\000\000\000\003\001\000\001')  # aka 65537 aka 0xf4
+                         b'\000\000\000\003\001\000\001')  # aka 65537 aka 0xf4
         with self.assertRaises(RSA.RSAError):
             setattr(rsa, 'e', '\000\000\000\003\001\000\001')
         with self.assertRaises(RSA.RSAError):
@@ -167,10 +173,41 @@ class RSATestCase(unittest.TestCase):
 
     def test_set_bn(self):
         rsa = RSA.load_pub_key(self.pubkey)
-        self.assertIsNone(m2.rsa_set_e(rsa.rsa,
-                          '\000\000\000\003\001\000\001'))
         with self.assertRaises(RSA.RSAError):
-            m2.rsa_set_e(rsa.rsa, '\000\000\000\003\001')
+            m2.rsa_set_en(rsa.rsa,
+                b'\000\000\000\003\001\000\001',
+                b'\000\000\000\003\001')
+
+    def test_set_n(self):
+        rsa = m2.rsa_new()
+        m2.rsa_set_n(rsa, b'\000\000\000\003\001\000\001')
+
+        n = m2.rsa_get_n(rsa)
+        e = m2.rsa_get_e(rsa)
+
+        self.assertEqual(n, b'\000\000\000\003\001\000\001')
+        self.assertEqual(e, b'\x00\x00\x00\x00')
+
+    def test_set_e(self):
+        rsa = m2.rsa_new()
+        m2.rsa_set_e(rsa, b'\000\000\000\003\001\000\001')
+
+        n = m2.rsa_get_n(rsa)
+        e = m2.rsa_get_e(rsa)
+
+        self.assertEqual(e, b'\000\000\000\003\001\000\001')
+        self.assertEqual(n, b'\x00\x00\x00\x00')
+
+    def test_set_n_then_set_e(self):
+        rsa = m2.rsa_new()
+        m2.rsa_set_n(rsa, b'\000\000\000\004\020\011\006\006')
+        m2.rsa_set_e(rsa, b'\000\000\000\003\001\000\001')
+
+        n = m2.rsa_get_n(rsa)
+        e = m2.rsa_get_e(rsa)
+
+        self.assertEqual(e, b'\000\000\000\003\001\000\001')
+        self.assertEqual(n, b'\000\000\000\004\020\011\006\006')
 
     def test_newpub(self):
         old = RSA.load_pub_key(self.pubkey)
@@ -178,7 +215,7 @@ class RSATestCase(unittest.TestCase):
         self.assertTrue(new.check_key())
         self.assertEqual(len(new), 1024)
         # aka 65537 aka 0xf4
-        self.assertEqual(new.e, '\000\000\000\003\001\000\001')
+        self.assertEqual(new.e, b'\000\000\000\003\001\000\001')
 
     def test_sign_and_verify(self):
         """
@@ -194,7 +231,7 @@ class RSATestCase(unittest.TestCase):
             algos['sha384'] = ''
             algos['sha512'] = ''
 
-        message = "This is the message string"
+        message = b"This is the message string"
         digest = hashlib.sha1(message).digest()
         rsa = RSA.load_key(self.privkey)
         rsa2 = RSA.load_pub_key(self.pubkey)
@@ -216,7 +253,7 @@ class RSATestCase(unittest.TestCase):
             size of the digest increases because of the size of
             our test key limits it.
             """
-            message = "This is the message string"
+            message = b"This is the message string"
             import hashlib
             algos = {'sha1': 43}
             if not fips_mode:
@@ -229,7 +266,7 @@ class RSATestCase(unittest.TestCase):
                 algos['sha384'] = 15
                 algos['sha512'] = 0
 
-            for algo, salt_max in algos.iteritems():
+            for algo, salt_max in algos.items():
                 h = hashlib.new(algo)
                 h.update(message)
                 digest = h.digest()
@@ -237,7 +274,7 @@ class RSATestCase(unittest.TestCase):
 
             rsa = RSA.load_key(self.privkey)
             rsa2 = RSA.load_pub_key(self.pubkey)
-            for algo, (salt_max, digest) in algos.iteritems():
+            for algo, (salt_max, digest) in algos.items():
                 for salt_length in range(0, salt_max):
                     signature = rsa.sign_rsassa_pss(digest, algo, salt_length)
                     verify = rsa2.verify_rsassa_pss(digest, signature,
@@ -251,7 +288,6 @@ class RSATestCase(unittest.TestCase):
         Testing calling sign with an unsupported message digest algorithm
         """
         rsa = RSA.load_key(self.privkey)
-        #message = "This is the message string"
         digest = 'a' * 16
         with self.assertRaises(ValueError):
                 rsa.sign(digest, 'bad_digest_method')
@@ -261,8 +297,7 @@ class RSATestCase(unittest.TestCase):
         Testing calling verify with an unsupported message digest algorithm
         """
         rsa = RSA.load_key(self.privkey)
-        #message = "This is the message string"
-        digest = 'a' * 16
+        digest = b'a' * 16
         signature = rsa.sign(digest, 'sha1')
         with self.assertRaises(ValueError):
             rsa.verify(digest, signature, 'bad_digest_method')
@@ -273,10 +308,9 @@ class RSATestCase(unittest.TestCase):
         message digest algorithm
         """
         rsa = RSA.load_key(self.privkey)
-        message = "This is the message string"
+        message = b"This is the message string"
         digest = hashlib.sha1(message).digest()
         signature = rsa.sign(digest, 'sha1')
-        #rsa2 = RSA.load_pub_key(self.pubkey)
         with self.assertRaises(RSA.RSAError):
             rsa.verify(digest, signature, 'md5')
 
@@ -288,7 +322,7 @@ class RSATestCase(unittest.TestCase):
         it has to be longer than a certain length.
         """
         rsa = RSA.load_key(self.privkey)
-        digest = """This string should be long enough to warrant an error in
+        digest = b"""This string should be long enough to warrant an error in
         RSA_sign""" * 2
 
         with self.assertRaises(RSA.RSAError):
@@ -299,10 +333,10 @@ class RSATestCase(unittest.TestCase):
         Testing verify to make sure it fails when we use a bad signature
         """
         rsa = RSA.load_key(self.privkey)
-        message = "This is the message string"
+        message = b"This is the message string"
         digest = hashlib.sha1(message).digest()
 
-        other_message = "Abracadabra"
+        other_message = b"Abracadabra"
         other_digest = hashlib.sha1(other_message).digest()
         other_signature = rsa.sign(other_digest)
 
